"""file to hold all files related to running top on Mac"""

# needed imports
import psutil
from datetime import datetime
from hurry.filesize import size

# create objects
now = datetime.now()


def upper_diagnostics():
    """Function to calculate all 'upper' diagnostics."""

    # processes
    processes = list(psutil.process_iter())

    # load average
    load_averages = psutil.getloadavg()

    # cpu usage
    cpu_usage = psutil.cpu_times()
    overall_usage = cpu_usage[0] + cpu_usage[2] + cpu_usage[3]

    # cannot get shared libs due to Mac, showing other stats instead
    cpu_count = psutil.cpu_count()
    phys_cpus = psutil.cpu_count(logical=False)

    # Cannot get memory regions due to having a Mac, showing other stats instead
    numUsers = len(psutil.users())
    bootTime = datetime.fromtimestamp(psutil.boot_time()).strftime("%Y-%m-%d %H:%M:%S")

    # physical memory
    vm = psutil.virtual_memory()

    # VM
    totalvm = psutil.virtual_memory().total
    swap = psutil.swap_memory()

    # Networks
    network_info = psutil.net_io_counters()

    # Disks
    disk_info = psutil.disk_io_counters()

    # return all of the metrics calculated in this function
    return (
        processes,
        load_averages,
        cpu_usage,
        overall_usage,
        cpu_count,
        phys_cpus,
        numUsers,
        bootTime,
        vm,
        totalvm,
        swap,
        network_info,
        disk_info,
    )


def process_stats():
    """Function to calculate order of processes in top command."""
    # create process list and initializations of needed func variables
    processes = []
    sleeping = 0
    running = 0
    threads = 0
    overallCPUPerc = 0

    # calculate the overall CPU percentage
    # over all processes psutil can grab
    for process in psutil.process_iter():
        try:
            overallCPUPerc += psutil.Process(process.pid).memory_info()[0] / 2.0**30
        except:
            overallCPUPerc += 0

    # calculate various statistics over every process
    # psutil can grab
    for process in psutil.process_iter():
        # PID
        id = process.pid
        # process name
        name = process.name()
        # CPU percent
        proc = psutil.Process(id)
        # status
        status = process.status()

        # get username that process is from
        user = process.username()

        try:
            # try to calculate the memory usage
            memoryUse = proc.memory_info()[0] / 2.0**30
        except:
            memoryUse = 0

        if process.status() == "sleeping":
            # count process if it is sleeping
            sleeping += 1
        elif process.status() == "running":
            # count process if it is running
            running += 1

        try:
            # count all threads from the process if able
            threads += process.num_threads()
        except:
            threads += 0

        # sum up memory usage and divide for CPU %
        cpuPerc = (memoryUse / overallCPUPerc) * 100

        # calculate the time that the process begin running
        seconds = process.create_time()
        start = datetime.fromtimestamp(seconds)

        # get the current time
        current = datetime.now()

        # calculate the difference between the process
        # start time and the current time
        difference = current - start

        # change the time difference into a usable format
        seconds = int(difference.total_seconds())
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)
        # concatenate together the calculated difference
        formatted_time = str(hours) + ":" + str(minutes) + ":" + str(seconds)

        # create a list in the list containing the calculated process info
        processes.append(
            [id, name[:12], round(cpuPerc, 2), formatted_time, user, status]
        )
    # order the list in descending order of cpu percentage
    processes.sort(key=lambda processes: processes[2], reverse=True)

    # return calculated metrics
    return processes, sleeping, running, threads


def top():
    """Function to compile all diagnostic information into a organized display."""
    # call needed function metrics
    (
        processes,
        load_averages,
        cpu_usage,
        overall_usage,
        cpu_count,
        phys_cpus,
        numUsers,
        bootTime,
        vm,
        totalvm,
        swap,
        network_info,
        disk_info,
    ) = upper_diagnostics()

    processes, sleeping, running, threads = process_stats()

    # create now variable for accurate timekeeping
    newnow = datetime.now()

    print(
        f"Processes: {len(processes)} total, {running} running, {sleeping} sleeping, {threads} threads",
        "{:>20}".format(newnow.strftime("%H:%M:%S")),
    )
    print(
        f"Load Avg: {round(load_averages[0], 2)}, {round(load_averages[1], 2)}, {round(load_averages[2], 2)}  CPU usage: {round((cpu_usage[0]/overall_usage) * 100, 2)}% user, {round((cpu_usage[2]/overall_usage) * 100,2)}% sys, {round((cpu_usage[3]/overall_usage) * 100,2)}% idle"
    )
    print(
        f"CPU counts: {cpu_count} total, {phys_cpus} physical  Users: {numUsers}  Boot Time: {bootTime}"
    )
    print(f"PhysMem: {size(vm[0])} used, {size(vm[7])} wired, {vm[5]} inactive.")
    print(f"VM: {size(totalvm)} vsize, {swap[4]}(0) swapins, {swap[5]}(0) swapouts")
    print(
        f"Networks: packets: {network_info[3]}/{size(network_info[3])} in, {network_info[2]}/{size(network_info[2])} out."
    )
    print(
        f"Disks: {disk_info[0]}/{size(disk_info[0])} read, {disk_info[1]}/{size(disk_info[1])} written"
    )

    # lower section print
    print("\nPID       COMMAND    CPU%   TIME       USER     STATUS")
    for i in range(14):
        proc = processes[i]
        print("{: <7} {: <12} {: <5} {: <8} {: <11} {: <10}".format(*proc))



"""file to hold all files related to running top on Windows"""

# needed imports
import psutil
from datetime import datetime
from hurry.filesize import size

# create objects
now = datetime.now()


def upper_diagnostics():
    """This function calculates all 'upper' diagnostics"""

    # load average
    load_avg = psutil.getloadavg()

    # cpu usage
    # usage
    cpu_usage = psutil.cpu_times()
    # overall usage
    overall_usage = cpu_usage[0] + cpu_usage[2] + cpu_usage[3]

    # memory regions: * only rss available on windows
    p = psutil.Process()
    mem = p.memory_maps()
    mem_regions = 0
    for i in range(len(mem)):
        mem_regions += mem[i][1]

    # VM ** only total and available on windows
    vm = psutil.virtual_memory()

    # swap memory ** do not use sin sout on windows, always will be 0
    swap = psutil.swap_memory()

    # networks
    network_info = psutil.net_io_counters()

    # disks
    disk_info = psutil.disk_io_counters()

    # return all metrics calculated
    return (
        load_avg,
        cpu_usage,
        overall_usage,
        mem_regions,
        vm,
        swap,
        network_info,
        disk_info,
    )


def process_stats():
    """This function calculates the order of processes in terms of CPU%"""

    # create process list and initializations of needed variables
    processes = []
    sleeping = 0
    running = 0
    totalproc = 0
    threads = 0
    overallCPUPerc = 0

    # get total processes number
    totalproc = len(list(psutil.process_iter()))

    # calculate overall CPU percentage
    # over all processes psutil can grab
    for process in psutil.process_iter():
        try:
            overallCPUPerc += psutil.Process(process.pid).memory_info()[0] / 2.0**30
        except:
            overallCPUPerc += 0

    # iterate through all processes and grab info for all
    for process in psutil.process_iter():
        memoryUse = 0
        cpuPerc = 0

        # pid
        id = process.pid

        # process name
        name = process.name()

        # status
        status = process.status()

        # username
        user = process.username()

        # calculate CPU perc
        try:
            proc = psutil.Process(id)
            memoryUse = proc.memory_info()[0] / 2.0**30
            cpuPerc = (memoryUse / overallCPUPerc) * 100
        except:
            pass

        # sleeping or running
        if process.status() == "sleeping":
            # count process if it is sleeping
            sleeping += 1
        elif process.status() == "running":
            # count process if it is running
            running += 1

        # threads
        try:
            # count all threads from the process if able
            threads += process.num_threads()
        except:
            threads += 0

        # time running
        start = datetime.fromtimestamp(process.create_time())
        current = datetime.now()

        difference = current - start

        # change the time difference into a usable format
        seconds = int(difference.total_seconds())
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)
        # concatenate together the calculated difference
        formatted_time = str(hours) + ":" + str(minutes) + ":" + str(seconds)

        # add process info to process list
        # create a list in the list containing the calculated process info
        processes.append(
            [id, name[:12], round(cpuPerc, 2), formatted_time, user, status]
        )

    # order the list in descending order of cpu percentage
    processes.sort(key=lambda processes: processes[2], reverse=True)

    # return list and other metrics
    return processes, totalproc, sleeping, running, threads


def top():
    """This function compiles all diagnostic info and prints it to the terminal"""

    updatedTime = datetime.now()

    (
        load_avg,
        cpu_usage,
        overall_usage,
        mem_regions,
        vm,
        swap,
        network_info,
        disk_info,
    ) = upper_diagnostics()

    processes, totalproc, sleeping, running, threads = process_stats()

    print(
        f"Processes: {totalproc} total, {running} running, {sleeping} sleeping, {threads} threads",
        "{:>20}".format(updatedTime.strftime("%H:%M:%S")),
    )
    print(
        f"Load Avg: {round(load_avg[0], 2)}, {round(load_avg[1], 2)}, {round(load_avg[2], 2)}  CPU usage: {round((cpu_usage[0]/overall_usage) * 100, 2)}% user, {round((cpu_usage[2]/overall_usage) * 100,2)}% sys, {round((cpu_usage[3]/overall_usage) * 100,2)}% idle"
    )
    print(f"MemRegions: {mem_regions} resident")
    print(f"PhysMem: {size(vm[0])} total, {size(vm[1])} available")
    print(f"VM: {size(vm[0])} vsize, {size(vm[2])} used, {size(vm[3])} free")
    print(f"Networks: packets: {network_info[2]} in, {network_info[3]} out")
    print(f"Disks: {disk_info[0]} read, {disk_info[1]} written")

    # lower section print
    print("\nPID       COMMAND    CPU%   TIME       USER     STATUS")
    for i in range(14):
        proc = processes[i]
        print("{: <7} {: <12} {: <5} {: <8} {: <11} {: <10}".format(*proc))



""" Program in order to create a 'top' command implemented in Python. """

# needed imports
import schedule
import time
import sys
import platform

# needed cross platform files
import mac
import windows
import linux


def main():

    operatingSys = platform.system()

    if operatingSys == "Darwin":
        if len(sys.argv) == 1:
            try:
                # if they want to run it iteratively
                schedule.every(1).seconds.do(mac.top)
                while True:
                    schedule.run_pending()
                    time.sleep(1)
            except KeyboardInterrupt:
                sys.exit(0)
        else:
            iterations = int(sys.argv[1])
            # run for the amount of times entered
            for i in range(iterations):
                mac.top()
                time.sleep(1)

    elif operatingSys == "Windows":
        if len(sys.argv) == 1:
            try:
                # if they want to run it iteratively
                schedule.every(1).seconds.do(windows.top)
                while True:
                    schedule.run_pending()
                    time.sleep(1)
            except KeyboardInterrupt:
                sys.exit(0)
        else:
            iterations = int(sys.argv[1])
            # run for the amount of times entered
            for i in range(iterations):
                windows.top()
                time.sleep(1)

    elif operatingSys == "Linux":
        if len(sys.argv) == 1:
            try:
                # if they want to run it iteratively
                schedule.every(1).seconds.do(linux.top)
                while True:
                    schedule.run_pending()
                    time.sleep(1)
            except KeyboardInterrupt:
                sys.exit(0)
        else:
            iterations = int(sys.argv[1])
            # run for the amount of times entered
            for i in range(iterations):
                linux.top()
                time.sleep(1)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ostop']

package_data = \
{'': ['*']}

install_requires = \
['datetime>=4.7,<5.0',
 'hurryfilesize>=0.9,<0.10',
 'psutil>=5.9.3,<6.0.0',
 'schedule>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['my_package_cli = ostop.main:main']}

setup_kwargs = {
    'name': 'ostop',
    'version': '1.4',
    'description': 'Cross-Compatible Python implementation of top command.',
    'long_description': "# top\n\n[![build](../../actions/workflows/build.yml/badge.svg)](../../actions/)\n[![Language: Python](https://img.shields.io/badge/Language-Python-blue.svg)](https://www.python.org/)\n[![Commits: Conventional](https://img.shields.io/badge/Commits-Conventional-blue.svg)](https://www.conventionalcommits.org/en/v1.0.0/)\n\nA Cross-Platform Python implementation of 'top' command using Psutil.\n\n## What top Does\n\nTop is able to get most of the top information with the\nrestrictions that come with running it at the program level.\nDifferent statistics are shown based on\nthe operating system that this project is run on.\ntop works on MacOS, Linux, and Windows operating systems.\n\n## How to Get Started With top\n\nYou can get started with top by cloning the repository and running this command:\n\n``` python src/top.py ```\n\nin the base directory.\nLike the top command, this will run forever.\nYou can exit out of the program by entering a keyboard\ninterrupt or exiting your terminal altogether.\nYou can also specify the amount of times you want the program\nto run by giving a second integer input.\nFor example, you can run the program for one iteration\nby writing the command\n\n```python src/top.py 1```.\n\n## Running GatorGrade Checks\n\nThis repository is able to be automatically assessed using GatorGrade.\nThese checks can be run from the repository's base directory by running the command\n\n```gatorgrade --config config/gatorgrade.yml```\n\nin the base directory if you have GatorGrade installed.\nIf you do not have GatorGrade installed yet on your local machine,\nyou can install it by using the command\n\n```pip install gatorgrade```.\n\nThese checks ensure that files are formatted correctly\nwith proficient levels of polish and also run without crashing.\nGatorGrade checks are  useful both during and after development.\n",
    'author': 'Katherine Burgess',
    'author_email': '20burgessk@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/burgess01/top',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
